@@ -27,9 +27,11 @@ typedef struct Stream_t {
 
 #include "mtools.h"
 #include "msdos.h"
-
+#include "device.h"
 #include "llong.h"
 
+void limitSizeToOffT(size_t *len, mt_off_t maxLen);
+
 doscp_t *get_dosConvert_pass_through(Stream_t *Stream);
 
 typedef struct Class_t {
@@ -37,9 +39,8 @@ typedef struct Class_t {
 	ssize_t (*write)(Stream_t *, char *, mt_off_t, size_t);
 	int (*flush)(Stream_t *);
 	int (*freeFunc)(Stream_t *);
-	int (*set_geom)(Stream_t *, device_t *, device_t *, int media,
-					union bootsector *);
-	int (*get_data)(Stream_t *, time_t *, mt_size_t *, int *, uint32_t *);
+	int (*set_geom)(Stream_t *, device_t *, device_t *);
+	int (*get_data)(Stream_t *, time_t *, mt_off_t *, int *, uint32_t *);
 	int (*pre_allocate)(Stream_t *, mt_off_t);
 
 	doscp_t *(*get_dosConvert)(Stream_t *);
@@ -53,8 +54,8 @@ typedef struct Class_t {
 #define WRITES(stream, buf, address, size) \
 ((stream)->Class->write)( (stream), (char *) (buf), (address), (size) )
 
-#define SET_GEOM(stream, dev, orig_dev, media, boot) \
-(stream)->Class->set_geom( (stream), (dev), (orig_dev), (media), (boot) )
+#define SET_GEOM(stream, dev, orig_dev) \
+(stream)->Class->set_geom( (stream), (dev), (orig_dev))
 
 #define GET_DATA(stream, date, size, type, address) \
 (stream)->Class->get_data( (stream), (date), (size), (type), (address) )
@@ -87,27 +88,27 @@ copy_stream( (stream) )
 ssize_t force_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len);
 ssize_t force_read(Stream_t *Stream, char *buf, mt_off_t start, size_t len);
 
-int set_geom_pass_through(Stream_t *Stream, device_t *dev,
-			  device_t *orig_dev, int media,
-			  union bootsector *boot);
+int set_geom_pass_through(Stream_t *Stream, device_t *dev, device_t *orig_dev);
 
-int get_data_pass_through(Stream_t *Stream, time_t *date, mt_size_t *size,
-						  int *type, uint32_t *address);
+int set_geom_noop(Stream_t *Stream, device_t *dev, device_t *orig_dev);
+
+int get_data_pass_through(Stream_t *Stream, time_t *date, mt_off_t *size,
+			  int *type, uint32_t *address);
 
 ssize_t read_pass_through(Stream_t *Stream, char *buf,
 			  mt_off_t start, size_t len);
 ssize_t write_pass_through(Stream_t *Stream, char *buf,
 			   mt_off_t start, size_t len);
 
-mt_off_t sectorsToBytes(Stream_t *This, uint32_t off);
-
 mt_off_t getfree(Stream_t *Stream);
-int getfreeMinBytes(Stream_t *Stream, mt_size_t ref);
+int getfreeMinBytes(Stream_t *Stream, mt_off_t ref);
 
 Stream_t *find_device(char drive, int mode, struct device *out_dev,
 		      union bootsector *boot,
-		      char *name, int *media, mt_size_t *maxSize,
+		      char *name, int *media, mt_off_t *maxSize,
 		      int *isRop);
 
+int adjust_tot_sectors(struct device *dev, mt_off_t offset, char *errmsg);
+
 #endif
 
